local BG = LibStub("AceAddon-3.0"):GetAddon("Battleground")
local module = BG:NewModule("ArathiBasin", "AceEvent-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("Battleground")

local landmarks = BG.landmarks

local pois = {
	left = {
		L["Stables"],
		L["Gold Mine"],
		L["Blacksmith"],
		L["Lumber Mill"],
		L["Farm"]
	},
	right = {
	},
	top = {
		L["Battleground Start"],
	},
	bottom = {
	}
}

local frame


local function OnUpdateTime(self, elapsed)
	if self.expire then
		local telapsed = time() - self.expire
		local m = floor(telapsed/60)
		local s = telapsed % 60
		self.time:SetFormattedText("%0d:%02d", m, s)
	end
end

local function OnUpdate(self, elapsed)
	if not self.expire then return end
	if self.expire >= time() then
		local tleft = self.expire - time()
		local m = floor(tleft/60)
		local s = tleft % 60
		self.time:SetFormattedText("%0d:%02d", m, s)
	elseif self.switchname then
		self.text:SetText(self.switchname)
		self.switchname = nil
		self.expire = time()
		self:SetScript("OnUpdate", OnUpdateTime)
	end
end


local function createGUI()
	if frame then return end
	frame = CreateFrame("Frame", nil, UIParent)
	frame:Hide()
	frame.pois = {}
	frame:SetPoint("TOP", WorldStateAlwaysUpFrame, "TOP", 30, -8) -- -66)
	frame:SetWidth(260)
	frame:SetHeight(30)
	local last
	for k, v in pairs( pois.left ) do
		frame.pois[v] = BG.getButton(frame, "left")
		frame.pois[v].text:SetText(v)
		if k == 1 then
			last = frame.pois[v]
			frame.pois[v]:SetPoint("RIGHT", frame, "LEFT")
		else
			frame.pois[v]:SetPoint("TOP", last, "BOTTOM")
			last = frame.pois[v]
		end
	end
	for k, v in pairs( pois.right ) do
		frame.pois[v] = BG.getButton(frame, "right")
		frame.pois[v].text:SetText(v)
		if k == 1 then
			last = frame.pois[v]
			frame.pois[v]:SetPoint("RIGHT", frame, "RIGHT")
		else
			frame.pois[v]:SetPoint("TOP", last, "BOTTOM")
			last = frame.pois[v]
		end
	end
	for k, v in pairs(pois.top) do
		frame.pois[v] = BG.getButton(frame, "right")
		frame.pois[v].text:SetText(v)
		if k == 1 then
			last = frame.pois[v]
			frame.pois[v]:SetPoint("BOTTOM", frame, "TOP", -50, 0)
		else
			frame.pois[v]:SetPoint("BOTTOM", last, "TOP")
			last = frame.pois[v]
		end
	end
	for k, v in pairs(pois.bottom) do
		frame.pois[v] = BG.getButton(frame, "right")
		frame.pois[v].text:SetText(v)
		if k == 1 then
			last = frame.pois[v]
			frame.pois[v]:SetPoint("TOP", frame, "TOP", -50, -56)
		else
			frame.pois[v]:SetPoint("TOP", last, "BOTTOM")
			last = frame.pois[v]
		end
	end
end

local function showGUI()
	if not frame then createGUI() end
	for k, v in pairs(frame.pois) do
		v.status = nil
	end
	frame.pois[L["Battleground Start"]].icon:SetTexture("Interface\\CharacterFrame\\UI-StateIcon")
	frame.pois[L["Battleground Start"]].icon:SetTexCoord(0.57, 0.90, 0.08, 0.41)
	frame:Show()
end

local function hideGUI()
	frame:Hide()
end

function module:OnEnable()
	showGUI()
	self:RegisterMessage("Battleground_POIUpdate")
	self:RegisterMessage("Battleground_StartTimer")
	self:RegisterMessage("Battleground_Started")
	self:RegisterEvent("CHAT_MSG_ADDON")
	-- request sync from as many as we can
	SendAddonMessage("cap", "r", "BATTLEGROUND") -- Capping request for timers
end

function module:OnDisable()
	hideGUI()
end

function module:Battleground_Started(message, t)
	local poi = frame.pois[L["Battleground Start"]]
	poi.expire = time() - t
	poi.text:SetText(L["Battleground Elapsed"])
	poi:SetScript("OnUpdate", OnUpdateTime)
end

function module:Battleground_StartTimer(message, t)
	local poi = frame.pois[L["Battleground Start"]]
	poi.text:SetText(L["Battleground Start"])
	poi.switchname = L["Battleground Elapsed"]
	poi.expire = time() + t
	poi:SetScript("OnUpdate", OnUpdate)
end

function module:Battleground_POIUpdate(message, name, ti)
	if not frame or not frame.pois[name] then return end
	local poi = frame.pois[name]
	poi.icon:SetTexCoord( landmarks[ti][3], landmarks[ti][4], landmarks[ti][5], landmarks[ti][6])
	local status = landmarks[ti][2]
	poi:SetAlpha(1)
	poi.text:SetTextColor(.6,.6,.6)
	if status == "horde" then
		local color = ChatTypeInfo["BG_SYSTEM_HORDE"]
		poi.text:SetTextColor(color.r, color.g, color.b)
	elseif status == "alliance" then
		local color = ChatTypeInfo["BG_SYSTEM_ALLIANCE"]
		poi.text:SetTextColor(color.r, color.g, color.b)
	end
	if ( status == "alliance_assaulted" or status == "horde_assaulted" ) and poi.status then
		poi.expire = time() + 62
		local color = ChatTypeInfo["BG_SYSTEM_HORDE"]
		if status == "alliance_assaulted" then color = ChatTypeInfo["BG_SYSTEM_ALLIANCE"] end
		poi.time:SetTextColor(color.r, color.g, color.b)
		poi:SetScript("OnUpdate", OnUpdate)
	else
		poi.status = status
		poi.expire = nil
		poi:SetScript("OnUpdate", nil)
		poi.time:SetText(nil)
	end
end

function module:CHAT_MSG_ADDON(event, prefix, message, chan, sender)
	if prefix == "cap" and message ~= "r" then
		local name, duration, elapsed, color = strmatch(message, "^(.+)@(%d+)@(%d+)@(%a+)$")
		local poi = frame.pois[name]
		if poi and not poi.expire then
			poi.expire = time() + (duration - elapsed)
			local color = ChatTypeInfo["BG_SYSTEM_HORDE"]
			if poi.status == "alliance_assaulted" then color = ChatTypeInfo["BG_SYSTEM_ALLIANCE"] end
			poi.time:SetTextColor(color.r, color.g, color.b)
			poi:SetScript("OnUpdate", OnUpdate)
		end
	else
		-- print(chan, prefix, sender, message)
	end
end
